﻿
using System;
using System.Collections.Generic;
using System.Linq;
using System.Windows.Forms;
using HeuristicLab.Data.Views;
using HeuristicLab.MainForm;
using HeuristicLab.MainForm.WindowsForms;

namespace HeuristicLab.DataPreprocessing {
  [View("Data Grid Content View")]
  [Content(typeof(IDataGridContent), true)]
  public partial class DataGridContentView : StringConvertibleMatrixView {

    private int lastClickedRow;
    private int lastClickedColumn;

    public new IDataGridContent Content {
      get { return (IDataGridContent)base.Content; }
      set { base.Content = value; }
    }

    public DataGridContentView() {
      InitializeComponent();
      dataGridView.CellMouseClick += dataGridView_CellMouseClick;
      contextMenuCell.Items.Add(ShowHideColumns);
    }

    protected override void OnContentChanged() {
      base.OnContentChanged();
    }


    private void dataGridView_CellValidating(object sender, DataGridViewCellValidatingEventArgs e) {
      if (!dataGridView.ReadOnly) {
        string errorMessage;
        if (Content != null && !Content.DataGridLogic.Validate(e.FormattedValue.ToString(), out errorMessage, e.ColumnIndex)) {
          e.Cancel = true;
          dataGridView.Rows[e.RowIndex].ErrorText = errorMessage;
        }
      }
    }

    protected override void SetEnabledStateOfControls() {
      base.SetEnabledStateOfControls();
      rowsTextBox.ReadOnly = true;
      columnsTextBox.ReadOnly = true;
    }

    private void btnApplySort_Click(object sender, System.EventArgs e) {
      Content.PreprocessingDataManipulation.ReOrderToIndices(virtualRowIndices);
      OnContentChanged();
    }

    private void dataGridView_CellMouseClick(object sender, DataGridViewCellMouseEventArgs e) {
      if (Content == null) return;
      if (e.Button == System.Windows.Forms.MouseButtons.Right) {
        if (e.ColumnIndex == -1 || e.RowIndex == -1) {
          contextMenu.Show(MousePosition);
        } else {
          interpolationToolStripMenuItem.Enabled = !(e.RowIndex == 0 || e.RowIndex == Content.Rows);
          contextMenuCell.Show(MousePosition);
          lastClickedColumn = e.ColumnIndex;
          lastClickedRow = e.RowIndex;
        }
      }
    }

    protected override int[] Sort(IEnumerable<KeyValuePair<int, SortOrder>> sortedColumns) {
      btnApplySort.Enabled = sortedColumns.Any();
      return base.Sort(sortedColumns);
    }

    protected override void ClearSorting() {
      btnApplySort.Enabled = false;
      base.ClearSorting();
    }

    private void ReplaceWithAverage_Click(object sender, EventArgs e) {
      Content.PreprocessingDataManipulation.ReplaceIndicesByAverageValue(lastClickedColumn, new List<int>() { lastClickedRow });
      OnContentChanged();
    }

    private void ReplaceWithMedian_Click(object sender, EventArgs e) {
      Content.PreprocessingDataManipulation.ReplaceIndicesByMedianValue(lastClickedColumn, new List<int>() { lastClickedRow });
      OnContentChanged();
    }

    private void ReplaceWithRandom_Click(object sender, EventArgs e) {
      Content.PreprocessingDataManipulation.ReplaceIndicesByRandomValue(lastClickedColumn, new List<int>() { lastClickedRow });
      OnContentChanged();
    }

    private void ReplaceWithMostCommon_Click(object sender, EventArgs e) {
      Content.PreprocessingDataManipulation.ReplaceIndicesByMostCommonValue(lastClickedColumn, new List<int>() { lastClickedRow });
      OnContentChanged();
    }

    private void ReplaceWithInterpolation_Click(object sender, EventArgs e) {
      Content.PreprocessingDataManipulation.ReplaceIndicesByLinearInterpolationOfNeighbours(lastClickedColumn, new List<int>() { lastClickedRow });
      OnContentChanged();
    }
  }
}
